/*
 * Copyright (c) 1994 Vectaport Inc., Cartoactive Systems
 *
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee, provided
 * that the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation, and that the names of the copyright holders not be used in
 * advertising or publicity pertaining to distribution of the software
 * without specific, written prior permission.  The copyright holders make
 * no representations about the suitability of this software for any purpose.
 * It is provided "as is" without express or implied warranty.
 *
 * THE COPYRIGHT HOLDERS DISCLAIM ALL WARRANTIES WITH REGARD TO THIS
 * SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS.
 * IN NO EVENT SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY SPECIAL,
 * INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING
 * FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT,
 * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
 * WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 * 
 */

#include <Dlg/dlgcov.h>
#include <Dlg/dlgdb.h>
#include <Dlg/dlgfclass.h>
#include <Dlg/dlgfeature.h>
#include <Dlg/dlglib.h>

#include <DlgUnidraw/dlgclasses.h>
#include <DlgUnidraw/dlgcmds.h>
#include <DlgUnidraw/dlgeditor.h>
#include <DlgUnidraw/dlgviewstate.h>

#include <MapUnidraw/mapcomps.h>
#include <MapUnidraw/mapstates.h>

#include <Map/mapprim.h>

#include <Unidraw/catalog.h>
#include <Unidraw/clipboard.h>
#include <Unidraw/selection.h>
#include <Unidraw/unidraw.h>
#include <Unidraw/viewer.h>

#include <Unidraw/Commands/edit.h>

#include <Unidraw/Graphic/graphic.h>

#include <InterViews/cursor.h>
#include <InterViews/transformer.h>
#include <InterViews/window.h>

#include <math.h>
#include <stdio.h>

/*****************************************************************************/

ClassId DlgShowCmd::GetClassId () { return DLGSHOW_CMD; }

boolean DlgShowCmd::IsA (ClassId id) {
    return DLGSHOW_CMD == id || Command::IsA(id);
}

DlgShowCmd::DlgShowCmd (ControlInfo* c) : Command(c) { }
DlgShowCmd::DlgShowCmd (Editor* ed) : Command(ed) { }

Command* DlgShowCmd::Copy () {
    Command* copy = new DlgShowCmd(CopyControlInfo());
    InitCopy(copy);
    return copy;
}

void DlgShowCmd::Execute () {
    DlgEditor* editor = (DlgEditor*)GetEditor();
    editor->GetWindow()->cursor(hourglass);
    GraphicComps* top = (GraphicComps*)GetEditor()->GetComponent();
    Clipboard* clb = new Clipboard();
    
    DlgDatabase* dlgdb = editor->GetMapDatabase();
    DlgViewState* viewstate = (DlgViewState*)editor->GetMapViewState();

    DlgLibrary* dlglib = dlgdb->library(editor->libstate()->maplib());
    if (!dlglib) return;

    DlgCoverage* dlgcov = dlglib->coverage(editor->covstate()->mapcov());
    if (!dlgcov) return;

    DlgFeatureClass* dlgfclass = dlgcov->feature_class(editor->fclassstate()->mapfclass());
    if (!dlgfclass) return;

    int nfeats = dlgfclass->nfeats();
    for (int i = 0; i<nfeats; i++) {
	MapFeature* feature = dlgfclass->feature(i);
	if (feature->face_primitive()->npts()<0) continue;
	if (!viewstate->themegs(feature)) continue;
	MapFeatureComp* mfc = new MapFeatureComp(feature);
	if (mfc) 
	    clb->Append(mfc);
    }
    
    PasteCmd* pastecmd = new PasteCmd(editor, clb);
    pastecmd->Execute();
    Selection* s = GetEditor()->GetViewer()->GetSelection();
    s->Clear();
}

boolean DlgShowCmd::Reversible () { return true; }


