/************************************************************************

  The binary header character functions

*************************************************************************/

#include "muse.h"

#if XVT_CC_PROTO
int
short_to_char(unsigned char *record, short *s, short big_endian, long *c)
#else

int
short_to_char(record, s, big_endian, c)
    unsigned char  *record;
    short          *s;
    short           big_endian;
    long           *c;
#endif

{
    long            i;
    union
    {				/* the short & character union          */
	unsigned short  s;
	unsigned char   c[2];
    }               shtchr;

    shtchr.s = *s;
    for (i = 0; i < 2; i++)
	if (big_endian)
	    record[(*c)++] = shtchr.c[1 - i];
	else
	    record[(*c)++] = shtchr.c[i];

    return STAT_SUCCESS;
}

#if XVT_CC_PROTO
int
ushort_to_char(unsigned char *record, unsigned short *s, short big_endian, long *c)
#else

int
ushort_to_char(record, s, big_endian, c)
    unsigned char  *record;
    unsigned short *s;
    short           big_endian;
    long           *c;
#endif

{
    long            i;
    union
    {				/* the short & character union          */
	unsigned short  s;
	unsigned char   c[2];
    }               shtchr;

    shtchr.s = *s;
    for (i = 0; i < 2; i++)
	if (big_endian)
	    record[(*c)++] = shtchr.c[1 - i];
	else
	    record[(*c)++] = shtchr.c[i];

    return STAT_SUCCESS;
}

#if XVT_CC_PROTO
int
long_to_char(unsigned char *record, long *l, short big_endian, long *c)
#else

int
long_to_char(record, l, big_endian, c)
    unsigned char  *record;
    long           *l;
    short           big_endian;
    long           *c;
#endif

{
    long            i;
    union
    {				/* the long & character union          */
	unsigned long   l;
	unsigned char   c[4];
    }               lngchr;

    lngchr.l = *l;
    for (i = 0; i < 4; i++)
	if (big_endian)
	    record[(*c)++] = lngchr.c[3 - i];
	else
	    record[(*c)++] = lngchr.c[i];

    return STAT_SUCCESS;
}

#if XVT_CC_PROTO
int
double_to_char(unsigned char *record, double *d, short big_endian, long *c)
#else

int
double_to_char(record, d, big_endian, c)
    unsigned char  *record;
    double         *d;
    short           big_endian;
    long           *c;
#endif

{
    long            i;
    union
    {				/* the double & character union         */

#if XVT_OS == XVT_OS_MAC
	short double    d;
#else
	double          d;
#endif
	unsigned char   c[8];
    }               dblchr;
    long            start;

    dblchr.d = *d;
    start = *c;

    for (i = 0; i < 8; i++)
	record[(*c)++] = dblchr.c[i];

    if (big_endian)
	convert_double(&record[start], (long) 4);

    return STAT_SUCCESS;
}

#if XVT_CC_PROTO
int
char_to_short(unsigned char *record, short *s, short big_endian, long *c)
#else

int
char_to_short(record, s, big_endian, c)
    unsigned char  *record;
    short          *s;
    short           big_endian;
    long           *c;
#endif

{
    long            i;
    union
    {				/* the short & character union          */
	unsigned short  s;
	unsigned char   c[2];
    }               shtchr;

    for (i = 0; i < 2; i++)
	if (big_endian)
	    shtchr.c[1 - i] = record[(*c)++];
	else
	    shtchr.c[i] = record[(*c)++];

    *s = shtchr.s;
    return STAT_SUCCESS;
}

#if XVT_CC_PROTO
int
char_to_ushort(unsigned char *record, unsigned short *s, short big_endian, long *c)
#else

int
char_to_ushort(record, s, big_endian, c)
    unsigned char  *record;
    unsigned short *s;
    short           big_endian;
    long           *c;
#endif

{
    long            i;
    union
    {				/* the short & character union          */
	unsigned short  s;
	unsigned char   c[2];
    }               shtchr;

    for (i = 0; i < 2; i++)
	if (big_endian)
	    shtchr.c[1 - i] = record[(*c)++];
	else
	    shtchr.c[i] = record[(*c)++];

    *s = shtchr.s;
    return STAT_SUCCESS;
}

#if XVT_CC_PROTO
int
char_to_long(unsigned char *record, long *l, short big_endian, long *c)
#else

int
char_to_long(record, l, big_endian, c)
    unsigned char  *record;
    long           *l;
    short           big_endian;
    long           *c;
#endif

{
    long            i;
    union
    {				/* the long & character union          */
	unsigned long   l;
	unsigned char   c[4];
    }               lngchr;

    for (i = 0; i < 4; i++)
	if (big_endian)
	    lngchr.c[3 - i] = record[(*c)++];
	else
	    lngchr.c[i] = record[(*c)++];

    *l = lngchr.l;
    return STAT_SUCCESS;
}

#if XVT_CC_PROTO
int
char_to_double(unsigned char *record, double *d, short big_endian, long *c)
#else

int
char_to_double(record, d, big_endian, c)
    unsigned char  *record;
    double         *d;
    short           big_endian;
    long           *c;
#endif

{
    long            i;
    union
    {				/* the double & character union         */

#if XVT_OS == XVT_OS_MAC
	short double    d;
#else
	double          d;
#endif
	unsigned char   c[8];
    }               dblchr;

    if (big_endian)
	convert_double(&record[(*c)], (long) 3);

    for (i = 0; i < 8; i++)
	dblchr.c[i] = record[(*c)++];

    *d = dblchr.d;
    return STAT_SUCCESS;
}
