/*
 * Copyright 1994, 1995 Vectaport Inc., Cartoactive Systems
 *
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee, provided
 * that the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation, and that the names of the copyright holders not be used in
 * advertising or publicity pertaining to distribution of the software
 * without specific, written prior permission.  The copyright holders make
 * no representations about the suitability of this software for any purpose.
 * It is provided "as is" without express or implied warranty.
 *
 * THE COPYRIGHT HOLDERS DISCLAIM ALL WARRANTIES WITH REGARD TO THIS
 * SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS.
 * IN NO EVENT SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY SPECIAL,
 * INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING
 * FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT,
 * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
 * WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 * 
 */

#include <Map/mapcov.h>
#include <Map/mapdb.h>
#include <Map/maplib.h>

#include <iostream.h>
#include <string.h>

/*****************************************************************************/

MapLibrary::MapLibrary (MapDatabase* db, const char* name) : MapFiles(name, nil) {
    database(db);
    _path = nil;
    _ncovs = -1;
    _coverages  = nil;
    _tiled = false;
}

MapLibrary::~MapLibrary () {
    for (int i=0; i<_ncovs; i++) 
	delete _coverages[i];
    delete [] _coverages;
}

const char* MapLibrary::path() const {
    if (_database && !_path) {
	const char* dbpath = _database->path();
	const int len = strlen(dbpath) + strlen(_name) + 2;
	((MapLibrary*)this)->_path = new char[len];
	char* libpath = ((MapLibrary*)this)->_path;
        strcpy( libpath, dbpath);
        strcat( libpath, "/" );
        strcat( libpath, _name );
    }
    return _path;
}

int MapLibrary::ncovs() const{
    if (!initialized()) ((MapLibrary*)this)->init_coverages();

    return _ncovs;
}

MapCoverage* MapLibrary::coverage(unsigned covnum) const {
    if (!initialized()) ((MapLibrary*)this)->init_coverages();
    
    if (covnum<_ncovs) 
	return _coverages[covnum];
    else
	return nil;
}
     
MapCoverage* MapLibrary::coverage(const char* covname) const {
    if (!initialized()) ((MapLibrary*)this)->init_coverages();
    
    for (int i = 0; i < _ncovs; i++) {
	if (strcmp(covname,_coverages[i]->name())==0)
	    return _coverages[i];
    }
    return nil;
}
     
const char* MapLibrary::description() const { return "generic library description"; }

ostream& operator<< (ostream& out, const MapLibrary& lib) {
    out << "Library:  " << lib.name() << "\n";
    out << "Path:  " << lib.path() << "\n";
    out << "Description:  " << lib.description() << "\n";
    out << "Coverages:  " << lib.ncovs() << "\n";
    for (int i = 0; i < lib.ncovs(); i++) {
	out << lib.coverage(i);
    }
    return out;
}

void MapLibrary::init_coverages() {
    if (initialized()) return;

    char **covnames = coverage_names(_ncovs);
    if (_ncovs==0) return;

    _coverages = new MapCoverage*[_ncovs];
    for (int i=0; i<_ncovs; i++) {
	_coverages[i] = make_coverage(covnames[i]);
	delete covnames[i];
    }
    delete [] covnames;
}

boolean MapLibrary::tiled() {
    return _tiled;
}

void MapLibrary::tiled(boolean t) {
    _tiled = t;
}
