/*
 * Copyright 1994, 1995 Vectaport Inc.
 * Copyright 1990, 1991 Stanford University 
 *
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee, provided
 * that the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation, and that the names of the copyright holders not be used in
 * advertising or publicity pertaining to distribution of the software
 * without specific, written prior permission.  The copyright holders make
 * no representations about the suitability of this software for any purpose.
 * It is provided "as is" without express or implied warranty.
 *
 * THE COPYRIGHT HOLDERS DISCLAIM ALL WARRANTIES WITH REGARD TO THIS
 * SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS.
 * IN NO EVENT SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY SPECIAL,
 * INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING
 * FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT,
 * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
 * WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 * 
 */

/*
 * MapExportCmd implementation.
 */

#include <MapUnidraw/mapexport.h>
#include <MapUnidraw/mapclasses.h>
#include <MapUnidraw/mapcomps.h>

#include <OverlayUnidraw/ovclasses.h>
#include <OverlayUnidraw/ovpsview.h>
#include <OverlayUnidraw/ovselection.h>
#include <OverlayUnidraw/ovviews.h>

#include <IVGlyph/exportchooser.h>
#include <IVGlyph/gdialogs.h>

#include <Unidraw/catalog.h>
#include <Unidraw/editor.h>
#include <Unidraw/iterator.h>
#include <Unidraw/unidraw.h>

#include <IV-look/dialogs.h>
#include <IV-look/kit.h>

#include <InterViews/session.h>
#include <InterViews/style.h>
#include <InterViews/window.h>

#include <OS/string.h>

#include <stream.h>
#include <stdio.h>
#include <stdlib.h>

/*****************************************************************************/

ClassId MapExportCmd::GetClassId () { return MAPEXPORT_CMD; }

boolean MapExportCmd::IsA (ClassId id) {
    return MAPEXPORT_CMD == id || OvExportCmd::IsA(id);
}

MapExportCmd::MapExportCmd (ControlInfo* c, ExportChooser* f) 
: OvExportCmd(c, f) { 
    _s = nil;
}

MapExportCmd::MapExportCmd (Editor* ed, ExportChooser* f)
: OvExportCmd(ed, f) {
    _s = nil;
}

MapExportCmd::~MapExportCmd () {
    delete _s;
}

Command* MapExportCmd::Copy () {
    MapExportCmd* copy = new MapExportCmd(CopyControlInfo());
    InitCopy(copy);
    return copy;
}

ExportScriptType MapExportCmd::type() {
    if (strcmp(format(), "idraw")==0) 
	return IdrawType;
    else if (strcmp(format(), "drawtool")==0)
	return OvIdrawType;
    else if (strcmp(format(), "mapdraw")==0)
	return MapIdrawType;
    else if (strcmp(format(), "dlgviewer")==0)
	return DlgIdrawType;
    else if (strcmp(format(), "vpfviewer")==0)
	return VpfIdrawType;
    else
	return MapIdrawType;
}

boolean MapExportCmd::Export (const char* pathname) {
    Editor* editor = GetEditor();
    MapIdrawComp* comp = (MapIdrawComp*)editor->GetComponent();
    boolean ok = false;
    AddParentsToSelection();

    ExternView *ev;
    if (chooser_->idraw_format())
	ev = (ExternView*) comp->Create(POSTSCRIPT_VIEW);
    else
	ev = (ExternView*) comp->Create(SCRIPT_VIEW);

    if (ev != nil) {
        filebuf fbuf;
	char* tmpfilename;
		
	if (chooser_->to_printer()) {
	    tmpfilename = tmpnam(nil);
	    ok = fbuf.open(tmpfilename, output) != 0;
	} else {
	    ok = fbuf.open(pathname, output) != 0;
	}

        if (ok) {
            ostream out(&fbuf);
            comp->Attach(ev);
	    ((OverlayPS*)ev)->SetCommand(this);
            ev->Update();
            ok = ev->Emit(out);
	    fbuf.close();

	    if (chooser_->to_printer()) {
		char cmd[CHARBUFSIZE];
		sprintf(cmd, "(%s %s;rm %s)&", pathname, tmpfilename, tmpfilename);
		ok = system(cmd) == 0;
	    }
        }
        delete ev;
    }
    return ok;
}

boolean MapExportCmd::Selected (GraphicView *view) {
    Iterator i;

    if (!_s->IsEmpty()) {
	_s->First(i);
	while (!_s->Done(i)) {
	    if (_s->GetView(i) == view) return true;
	    _s->Next(i);
	}
	return false;
    }
}

void MapExportCmd::AddParentsToSelection() {
    Editor* editor = GetEditor();
    Selection* s = editor->GetSelection();
    delete _s;
    _s = new OverlaySelection(s);
    Iterator i;

    if (!_s->IsEmpty()) {
	_s->First(i);
	while (!_s->Done(i)) {

	    /* add each parent to the list if not there yet */
	    GraphicView* parent = (GraphicView*)_s->GetView(i);
	    while (parent = (GraphicView*)parent->GetParent()) {
		Iterator j;
		_s->First(j);
		boolean notdone;
		while (notdone = !_s->Done(j)) {
		    if (_s->GetView(j) == parent) break;
		    _s->Next(j);
		}
		if (!notdone)
		    _s->Prepend(parent);
	    }
	    _s->Next(i);
	}
    }
}
