/*
 * Copyright 1997 Vectaport Inc.
 * Copyright 1994, 1995 Vectaport Inc., Cartoactive Systems
 *
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee, provided
 * that the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation, and that the names of the copyright holders not be used in
 * advertising or publicity pertaining to distribution of the software
 * without specific, written prior permission.  The copyright holders make
 * no representations about the suitability of this software for any purpose.
 * It is provided "as is" without express or implied warranty.
 *
 * THE COPYRIGHT HOLDERS DISCLAIM ALL WARRANTIES WITH REGARD TO THIS
 * SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS.
 * IN NO EVENT SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY SPECIAL,
 * INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING
 * FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT,
 * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
 * WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 * 
 */

#ifndef mapviews_h
#define mapviews_h

#include <MapUnidraw/mapcomps.h>

#include <FrameUnidraw/frameviews.h>

class FullGraphic;
class MapCoverage;
class MapFeatureClass;
class MapLibrary;
class MapEditor;

class MapFeatureView : public OverlayView {
public:
    MapFeatureView(MapFeatureComp* = nil);
    virtual ~MapFeatureView();

    virtual void Update();
    virtual Graphic* GetGraphic();

    virtual ClassId GetClassId();
    virtual boolean IsA(ClassId);

    virtual Graphic* HighlightGraphic();
    virtual void Highlight();
    virtual void Unhighlight();

protected:
    Graphic* _orig_gs;
    static FullGraphic* _afv_gs;
    static FullGraphic* _til_gs;
    static FullGraphic* _lfv_gs;
    static FullGraphic* _pfv_gs;
    static FullGraphic* _tfv_gs;
};

class MapFClassView : public OverlaysView {
public:
    MapFClassView(MapFClassComp* = nil);
    virtual ~MapFClassView();

    virtual void Interpret(Command*);
    virtual void Uninterpret(Command*);

    virtual ClassId GetClassId();
    virtual boolean IsA(ClassId);

    virtual void Update();
};

class MapCovView : public OverlaysView {
public:
    MapCovView(MapCovComp* = nil);
    virtual ~MapCovView();

    virtual void Interpret(Command*);
    virtual void Uninterpret(Command*);

    virtual ClassId GetClassId();
    virtual boolean IsA(ClassId);
    MapFClassView* FindFClassView(MapFeatureClass*);
};

class MapLibView : public OverlaysView {
public:
    MapLibView(MapLibComp* = nil);
    virtual ~MapLibView();

    virtual void Interpret(Command*);
    virtual void Uninterpret(Command*);

    virtual ClassId GetClassId();
    virtual boolean IsA(ClassId);
    MapCovView* FindCovView(MapCoverage*);
};

class MapDbView : public OverlaysView {
public:
    MapDbView(MapDbComp* = nil);
    virtual ~MapDbView();

    virtual void Interpret(Command*);
    virtual void Uninterpret(Command*);

    virtual ClassId GetClassId();
    virtual boolean IsA(ClassId);

    MapLibView* FindLibView(MapLibrary*);
};

class MapIdrawView : public FrameIdrawView {
public:
    MapIdrawView(MapIdrawComp* = nil);

    virtual ClassId GetClassId();
    virtual boolean IsA(ClassId);

    static void SetMapEditor(MapEditor* ed) { _ed = ed; }
    static MapEditor* GetMapEditor() {return _ed;}
protected:
    static MapEditor* _ed;
};

#endif
