/*
 * Copyright 1993, 1994 Vectaport, Jorge A. Gautier Consulting 
 * Copyright 1995 Vectaport Inc., Cider Press
 *
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee, provided
 * that the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation, and that the names of the copyright holders not be used in
 * advertising or publicity pertaining to distribution of the software
 * without specific, written prior permission.  The copyright holders make
 * no representations about the suitability of this software for any purpose.
 * It is provided "as is" without express or implied warranty.
 *
 * THE COPYRIGHT HOLDERS DISCLAIM ALL WARRANTIES WITH REGARD TO THIS
 * SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS.
 * IN NO EVENT SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY SPECIAL,
 * INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING
 * FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT,
 * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
 * WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 * 
 */

#include <Vpf/vpfdb.h>
#include <Vpf/vpflib.h>
#include <Vpf/vpfcov.h>

#include <VpfUnidraw/vpfclasses.h>
#include <VpfUnidraw/vpfcmds.h>
#include <VpfUnidraw/vpfcomps.h>
#include <VpfUnidraw/vpfcreator.h>
#include <VpfUnidraw/vpfscripts.h>
#include <VpfUnidraw/vpftilesel.h>

#include <OverlayUnidraw/ovclasses.h>
#include <OverlayUnidraw/ovpsview.h>

#include <Unidraw/catalog.h>

/*****************************************************************************/


VpfCreator::VpfCreator () { }

void* VpfCreator::Create (
    ClassId id, istream& in, ObjectMap* objmap, int objid
) {
    switch (id) {
        case VPFTOOLBAR_CMD:       CREATE(VpfToolbarCmd, in, objmap, objid);
        case VPFSHOWTILES_CMD:     CREATE(VpfShowTilesCmd, in,objmap,objid);
        case VPFOPEN_CMD:          CREATE(VpfOpenCmd, in, objmap, objid);
        case VPFUPDATEMENUS_CMD:   CREATE(VpfUpdateMenusCmd, in, objmap, objid);

        case TILESELECT_TOOL:      CREATE(TileSelectTool, in,objmap,objid);

        default:                   return MapCreator::Create(id, in,objmap,objid);
    }
}

void* VpfCreator::Create (ClassId id) {
    if (id == VPFFCLASS_VIEW)   return new VpfFClassView;
    if (id == VPFCOV_VIEW)      return new VpfCovView;
    if (id == VPF_IDRAW_VIEW)   return new MapIdrawView;

    if (id == VPFFCLASS_SCRIPT) return new VpfFClassScript;
    if (id == VPFCOV_SCRIPT)    return new VpfCovScript;
    if (id == VPFLIB_SCRIPT)    return new VpfLibScript;
    if (id == VPFDB_SCRIPT)     return new VpfDbScript;
    if (id == VPF_IDRAW_SCRIPT) return new VpfIdrawScript;

    if (id == VPFFCLASS_PS)     return new OverlaysPS;
    if (id == VPFCOV_PS)        return new OverlaysPS;
    if (id == VPF_IDRAW_PS)     return new OverlayIdrawPS;

    if (id == VPFFCLASS_RAW)    return new VpfFClassRawScript;
    return MapCreator::Create(id);
}
